import React, { useState, useRef, ChangeEvent } from "react";
import { getDownloadURL, ref, uploadBytesResumable } from "firebase/storage";
import moment from "moment";
import ReactPlayer from "react-player";
import { OnProgressProps } from "react-player/base";
import CommonAPI from "../../common/apis/common";
import { ApplicationPayloadType } from "../../common/types";
import Progress from "../Progress";
import { fileStorage } from "../../common/config";
import { useDispatch } from "react-redux";
import { updateApplication } from "../../common/actions/common";
import { getVerificationColor } from "../../common/lib/util";

type OwnProps = {
  application: ApplicationPayloadType;
};

type Analysis = {
  voice: string;
};

const Interview = ({ application }: OwnProps) => {
  const dispatch = useDispatch();
  const [analysis, setAnalysis] = useState<Analysis | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(false);
  const [startTime, setStartTime] = useState<number>(0);
  const [progress, setProgress] = useState<number>(0);
  const [endTime, setEndTime] = useState<number>(120);
  const _selectedTime = useRef<number>(0);
  const _video = useRef<ReactPlayer>(null);
  const _hiddenFileInput = useRef<HTMLInputElement>(null);

  const onSeek = (e: number) => (_selectedTime.current = e);
  const onProgress = (e: OnProgressProps) =>
    (_selectedTime.current = e.playedSeconds);
  const onSetStartTime = () => setStartTime(_selectedTime.current);
  const onSetEndTime = () => setEndTime(_selectedTime.current);

  const displayStartTime = `${moment.utc(startTime * 1000).format("HH:mm:ss")}`;
  const displayEndTime = `${moment.utc(endTime * 1000).format("HH:mm:ss")}`;

  const onClickAnalyse = () => {
    if (startTime !== endTime) {
      setIsLoading(true);
      CommonAPI.analyseInterview({
        startTime,
        endTime,
        applicationId: application._id,
      })
        .then((res: any) => {
          setAnalysis(res);
          setIsLoading(false);
        })
        .catch((err) => {
          setIsLoading(false);
        });
    }
  };
  const onSelectVideo = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.files) {
      setIsLoading(true);
      const path = `/interviews/${application._id}.mp4`;
      const uploadRef = ref(fileStorage, path);
      const uploadTask = uploadBytesResumable(uploadRef, e.target.files[0]);
      uploadTask.on(
        "state_changed",
        (snapshot) => {
          const percent = Math.round(
            (snapshot.bytesTransferred / snapshot.totalBytes) * 100
          );
          setProgress(percent);
        },
        (err) => {},
        () => {
          getDownloadURL(uploadTask.snapshot.ref).then((url) => {
            dispatch(
              updateApplication(
                {
                  applicationId: application._id,
                  candidateId: application.candidate._id || "",
                  update: { interview: { videoRef: url } },
                },
                () => {
                  setProgress(0);
                  setIsLoading(false);
                }
              )
            );
          });
        }
      );
    }
  };
  const onClickUpload = () => {
    _hiddenFileInput?.current?.click();
  };

  const renderLoading = isLoading && (
    <div className="spinner-border text-primary" role="status">
      <span className="visually-hidden">Loading...</span>
    </div>
  );

  if (!application.interview?.videoRef) {
    return (
      <>
        <div className="upload-interview-video">
          <button
            type="button"
            className="btn btn-secondary"
            onClick={onClickUpload}
            disabled={isLoading}
          >
            Upload video
          </button>
          <input
            type="file"
            accept="video/mp4"
            ref={_hiddenFileInput}
            onChange={onSelectVideo}
          />
        </div>
        <Progress progress={progress} />
      </>
    );
  }

  const renderVoiceVerification = () => {
    if (application.interview?.voiceVerification) {
      const score = application?.interview?.voiceVerification * 100;
      const percentage = `${score?.toFixed(2)}%`;

      return (
        <strong
          className={getVerificationColor(
            application?.interview?.voiceVerification * 100
          )}
        >
          Voice verification: {percentage}
        </strong>
      );
    }
    return null;
  };

  return (
    <div>
      <ReactPlayer
        ref={_video}
        url={application.interview.videoRef}
        controls
        height="100%"
        width="100%"
        onSeek={onSeek}
        onProgress={onProgress}
      />
      {renderVoiceVerification()}

      <div className="row" style={{ padding: "12px" }}>
        <div className="col-sm-4">
          <div className="row">
            <button
              type="button"
              className="col-sm-4 btn btn-warning btn-sm"
              onClick={onSetStartTime}
            >
              Start time
            </button>
            <div className="col-sm-8">
              <input
                type="text"
                className="form-control"
                name="startTime"
                disabled
                value={displayStartTime}
              />
            </div>
          </div>
        </div>
        <div className="col-sm-4">
          <div className="row">
            <button
              type="button"
              className="col-sm-4 btn btn-warning btn-sm"
              onClick={onSetEndTime}
            >
              End time
            </button>
            <div className="col-sm-8">
              <input
                type="text"
                className="form-control"
                name="endTime"
                disabled
                value={displayEndTime}
              />
            </div>
          </div>
        </div>
        <div className="col-sm-4">
          <div className="row">
            <div className="col-sm-8">
              <button
                type="button"
                className="btn btn-primary w-100"
                onClick={onClickAnalyse}
                disabled={isLoading}
              >
                Analyse
              </button>
            </div>
            {renderLoading}
          </div>
        </div>
      </div>

      <div className="row mt-2">
        <div className="mb-3 row">
          <label className="col-sm-2 col-form-label">Voice</label>
          <label className="col-sm-5 col-form-label">
            <strong>{analysis?.voice}</strong>
          </label>
        </div>
      </div>
    </div>
  );
};

export default Interview;
